/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER         "0.1"
#define BUFFSZ      1024
#define PWDCHK(x)   if(x == 0x1f) { \
                        x = *++pwd; \
                        if(x == 0x1f) { \
                            x = *++pwd - 0x22; \
                        } else x -= 0x21; \
                    } \
                    pwd++;
#define SESS        "Session="
#define USER        "UserName="
#define PASS        "Password="

#ifndef WIN32
    #define strnicmp strncasecmp
#endif
#ifndef u_char
    #define u_char  unsigned char
#endif
#ifndef u_short
    #define u_short unsigned short
#endif



void enternet_pwd(u_char *out, u_char *pwd);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  buff[BUFFSZ + 1],
            *ptr;


    setbuf(stdout, NULL);

    fputs("\n"
        "Enternet passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <EnterNet.ini>\n"
            "\n"
            "Enternet.ini is located in the Enternet installation folder, usually:\n"
            "\n"
            "    c:\\program files\\PROVIDER\\app\\EnterNet.ini\n"
            "\n"
            "where PROVIDER is \"usually\" the name of your DSL provider\n"
            "Drag'n'drop is highly suggested if you are in Windows\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("\nOpening file %s\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    fputc('\n', stdout);
    while(1) {
        if(!fgets(buff, BUFFSZ, fd)) break;

        if(!strnicmp(buff, SESS, sizeof(SESS) - 1)) {
            printf("\n%s", buff);
        } else if(!strnicmp(buff, USER, sizeof(USER) - 1)) {
            fputs(buff, stdout);
        } else if(!strnicmp(buff, PASS, sizeof(PASS) - 1)) {
            for(ptr = buff; *ptr > 0xd; ptr++);
            *ptr = 0;

            enternet_pwd(buff, buff + sizeof(PASS) - 1);
            printf(PASS"%s\n", buff);
        }
    }

    fclose(fd);
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void enternet_pwd(u_char *out, u_char *pwd) {
    u_char  al,
            bl,
            edx = 1;
    u_short esp;

    if(*pwd != 0x7f) {
        strcpy(out, pwd);
        return;
    }

    bl = *++pwd;
    PWDCHK(bl);
    while((al = *pwd)) {
        PWDCHK(al);
        edx &= 7;
        esp = ((al ^ bl) << 8) >> edx;
        *out++ = (esp >> 8) | (esp & 0xff);
        edx++;
    }
    *out = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


